﻿using System;
using System.Data;
using AutoMapper;
using MVCAndKnockout.Business.Models;
using MVCAndKnockout.Common;
using MVCAndKnockout.DataAccess.Repository;
using StructureMap;
using System.Collections.Generic;
using System.Linq;

namespace MVCAndKnockout.Business.Services
{
    public class ContactManagementService : IContactManagementService
    {
        private readonly IContactManagementRepository _contactManagementRepository;

        public ContactManagementService()
        {
            _contactManagementRepository = ObjectFactory.GetInstance<IContactManagementRepository>();
           
        }

        public void Dispose()
        {
            _contactManagementRepository.Dispose();
        }

        public IEnumerable<ContactType> GetContactTypes(ICriteria criteria)
        {
            return
                _contactManagementRepository.GetContactTypes(criteria)
                                            .Select(Mapper.Map<Data.ContactType, ContactType>);
        }

        public IEnumerable<ContactType> GetAllContactTypes()
        {
            return
                _contactManagementRepository.GetAllContactTypes()
                                            .Select(Mapper.Map<Data.ContactType, ContactType>);
        }

        public int GetTotalContactTypes()
        {
            return _contactManagementRepository.GetTotalContactTypes();
        }



        public IOperationResult<ContactType> CreateContactType(ContactType contactType)
        {
            try
            {
                return new OperationResult<ContactType>(Mapper.Map<Data.ContactType, ContactType>(_contactManagementRepository.Insert(Mapper.Map<ContactType, Data.ContactType>(contactType))), string.Empty, false, false);
            }
            catch (Exception exception)
            {
                // log exception
                return new OperationResult<ContactType>(contactType, string.Empty, false, true);
            }
        }

        public IOperationResult<ContactType> GetContactType(int id)
        {
            try
            {
                return new OperationResult<ContactType>(Mapper.Map<Data.ContactType, ContactType>(_contactManagementRepository.GetContactType(id)), string.Empty, false, false);
            }
            catch (Exception exception)
            {
                // log exception
                return new OperationResult<ContactType>(null, string.Empty, false, true);
            }
        }

        public IOperationResult<ContactType> UpdateContactType(ContactType contactType)
        {
            try
            {
                return new OperationResult<ContactType>(Mapper.Map<Data.ContactType, ContactType>(_contactManagementRepository.Update(Mapper.Map<ContactType, Data.ContactType>(contactType))), string.Empty, false, false);
            }
            catch (OptimisticConcurrencyException exception)
            {
                return new OperationResult<ContactType>(contactType, string.Empty, true, false);
            }
            catch (Exception exception)
            {
                // log exception
                return new OperationResult<ContactType>(contactType, string.Empty, false, true);
            }
        }

        public IOperationResult<ContactType> DeleteContactType(ContactType contactType)
        {
            try
            {
                return new OperationResult<ContactType>(Mapper.Map<Data.ContactType, ContactType>(_contactManagementRepository.Delete(Mapper.Map<ContactType, Data.ContactType>(contactType))), string.Empty, false, false);
            }
            catch (OptimisticConcurrencyException exception)
            {
                return new OperationResult<ContactType>(contactType, string.Empty, true, false);
            }
            catch (Exception exception)
            {
                // log exception
                return new OperationResult<ContactType>(contactType, string.Empty, false, true);
            }
        }


        public IOperationResult<Contact> CreateContact(Contact contact)
        {
            try
            {
                return new OperationResult<Contact>(Mapper.Map<Data.Contact, Contact>(_contactManagementRepository.Insert(Mapper.Map<Contact, Data.Contact>(contact))), string.Empty, false, false);
            }
            catch (Exception exception)
            {
                // log exception
                return new OperationResult<Contact>(contact, string.Empty, false, true);
            }
   
        }

        public IOperationResult<Contact> GetContact(int id)
        {
            try
            {
                return new OperationResult<Contact>(Mapper.Map<Data.Contact, Contact>(_contactManagementRepository.GetContact(id)), string.Empty, false, false);
            }
            catch (Exception exception)
            {
                // log exception
                return new OperationResult<Contact>(null, string.Empty, false, true);
            }
        }

        public IOperationResult<Contact> UpdateContact(Contact contact)
        {
            try
            {
                return new OperationResult<Contact>(Mapper.Map<Data.Contact, Contact>(_contactManagementRepository.Update(Mapper.Map<Contact, Data.Contact>(contact))), string.Empty, false, false);
            }
            catch (OptimisticConcurrencyException exception)
            {
                return new OperationResult<Contact>(contact, string.Empty, true, false);
            }
            catch (Exception exception)
            {
                // log exception
                return new OperationResult<Contact>(contact, string.Empty, false, true);
            }
        }

        public IOperationResult<Contact> DeleteContact(Contact contact)
        {
            try
            {
                return new OperationResult<Contact>(Mapper.Map<Data.Contact, Contact>(_contactManagementRepository.Delete(Mapper.Map<Contact, Data.Contact>(contact))), string.Empty, false, false);
            }
            catch (OptimisticConcurrencyException exception)
            {
                return new OperationResult<Contact>(contact, string.Empty, true, false);
            }
            catch (Exception exception)
            {
                // log exception
                return new OperationResult<Contact>(contact, string.Empty, false, true);
            }
        }


        public int GetTotalContacts()
        {
            return _contactManagementRepository.GetTotalContacts();
        }


        public IEnumerable<ContactInfo> GetContacts(ICriteria criteria)
        {
            return _contactManagementRepository.GetContacts(criteria)
                .Select(Mapper.Map<Data.ContactInfo, ContactInfo>); 
        }
    }
}