﻿using System;
using System.Globalization;
using System.Linq;
using System.Web.Mvc;
using MVCAndKnockout.Business.Models;
using MVCAndKnockout.Business.Services;
using MVCAndKnockout.Models.JqGrid;
using StructureMap;

namespace MVCAndKnockout.Controllers
{
    public class ContactController : ControllerBase
    {
        private readonly IContactManagementService _managementService = ObjectFactory.GetInstance<IContactManagementService>();

        public ActionResult Index()
        {
            return View();
        }

        public ActionResult Create()
        {
            CreateDropDown(0);
            ViewBag.Title = "Create";
            return View("Edit", new Contact());
        }


        public ActionResult Edit(int id)
        {
            var contact = _managementService.GetContact(id);
            CreateDropDown(contact.Result.ContactTypeId);
            ViewBag.Title = "Create";
            return View(contact.Result);
        }

        [HttpPost]
        public ActionResult Save(Contact contact)
        {
            if (ModelState.IsValid)
            {
                IOperationResult<Contact> result;
                if (contact.ContactId > 0)
                {
                    result = _managementService.UpdateContact(contact);
                }
                else
                {
                    result = _managementService.CreateContact(contact);
                }
                return Json(result);
            }
            return Json(new OperationResult<Contact>(contact, GetValidationErrors(), false, false));
        }


        [HttpPost]
        public ActionResult DeleteById(int id)
        {
            Contact contact = new Contact
                {
                    ContactId = id
                };
            var result = _managementService.DeleteContact(contact);
            return Json(result);
        }

        private void CreateDropDown(int id)
        {
            ViewData["ContactTypeId"] = CreateSelectList(_managementService.GetAllContactTypes().ToList(), id, i => i.ContactTypeId, i => i.Name);
        }

        public ActionResult GetContacts(JqGridQueryRequest criteria)
        {
            var totalRecords = _managementService.GetTotalContacts();
            var totalPages = (int)Math.Ceiling(totalRecords / (double)criteria.PageSize);
            var data = _managementService.GetContacts(criteria);
            var json = new JqGridData
            {
                page = criteria.PageIndex,
                records = totalRecords,
                total = totalPages,
                rows =
                    (from oneRow in data
                     select new JqGridRowData
                     {
                         id = oneRow.ContactId.ToString(CultureInfo.InvariantCulture),
                         cell = new[]
							{
                                oneRow.ContactId.ToString(CultureInfo.InvariantCulture),
                                oneRow.LastName,
                                oneRow.FirstName,
                                oneRow.ContactId.ToString(CultureInfo.InvariantCulture),
                                oneRow.ContactId.ToString(CultureInfo.InvariantCulture)
							}
                     }
                    ).ToArray()

            };
            return Json(json, JsonRequestBehavior.AllowGet);
        }

        protected override void Dispose(bool disposing)
        {
            _managementService.Dispose();
            base.Dispose(disposing);
        }

    }
}
