﻿using System;
using System.Globalization;
using System.Linq;
using System.Web.Mvc;
using MVCAndKnockout.Business.Models;
using MVCAndKnockout.Models.JqGrid;
using MVCAndKnockout.Business.Services;
using StructureMap;

namespace MVCAndKnockout.Controllers
{
    public class ContactTypeController : ControllerBase
    {
        private readonly IContactManagementService _managementService = ObjectFactory.GetInstance<IContactManagementService>();

        public ActionResult Index()
        {
            return View();
        }

        public ActionResult Create()
        {
            ViewBag.Title = "Create";
            return View("Edit", new ContactType());
        }

        [HttpPost]
        public ActionResult Save(ContactType contactType)
        {
            if (ModelState.IsValid)
            {
                IOperationResult<ContactType> result;
                if (contactType.ContactTypeId > 0)
                {
                    result = _managementService.UpdateContactType(contactType);    
                }
                else
                {
                    result = _managementService.CreateContactType(contactType);    
                }

                return Json(result);
            }
            return Json(new OperationResult<ContactType>(contactType, GetValidationErrors(), false, false));
        }


        public ActionResult Edit(int id)
        {
            ViewBag.Title = "Edit";
            return View(_managementService.GetContactType(id).Result);
        }


        [HttpPost]
        public ActionResult DeleteById(int id)
        {
            ContactType contactType = new ContactType
            {
                ContactTypeId = id
            };
            var result = _managementService.DeleteContactType(contactType);
            return Json(result);
        }

        public ActionResult GetTypes(JqGridQueryRequest criteria)
        {
            var totalRecords = _managementService.GetTotalContactTypes();
            var totalPages = (int)Math.Ceiling(totalRecords / (double)criteria.PageSize);
            var data = _managementService.GetContactTypes(criteria);
            var json = new JqGridData
            {
                page = criteria.PageIndex,
                records = totalRecords,
                total = totalPages,
                rows =
                    (from oneRow in data
                     select new JqGridRowData
                     {
                         id = oneRow.ContactTypeId.ToString(CultureInfo.InvariantCulture),
                         cell = new[]
							{
                                oneRow.ContactTypeId.ToString(CultureInfo.InvariantCulture),
                                oneRow.Name,
                                oneRow.ContactTypeId.ToString(CultureInfo.InvariantCulture),
                                oneRow.ContactTypeId.ToString(CultureInfo.InvariantCulture)
							}
                     }
                    ).ToArray()

            };
            return Json(json, JsonRequestBehavior.AllowGet);
        }

        protected override void Dispose(bool disposing)
        {
            _managementService.Dispose();
            base.Dispose(disposing);
        }

    }
}
