﻿using System.Linq.Expressions;

// ReSharper disable CheckNamespace to help coding inside views
namespace System.Web.Mvc.Html
// ReSharper restore CheckNamespace
{
    /// <summary>
    /// Custom editor extensions
    /// </summary>
    public static class EditorExtensions
    {
        /// <summary>
        /// Creates a custom editor based on template
        /// </summary>
        /// <typeparam name="TModel">The type of the model.</typeparam>
        /// <typeparam name="TValue">The type of the value.</typeparam>
        /// <param name="html">The HTML helper.</param>
        /// <param name="expression">The expression to evaluate.</param>
        /// <param name="editorTemplate">The editor template to use.</param>
        /// <param name="fieldName"> </param>
        /// <param name="htmlAttributes"> </param>
        /// <returns></returns>
        public static MvcHtmlString EditorFor<TModel, TValue>(
            this HtmlHelper<TModel> html, 
            Expression<Func<TModel, TValue>> expression, 
            EditorTemplate editorTemplate,
            string fieldName,
            object htmlAttributes)
        {
            return html.EditorFor(expression, editorTemplate.ToString(), fieldName, htmlAttributes);
        }

        /// <summary>
        /// Creates a custom editor based on template
        /// </summary>
        /// <typeparam name="TModel">The type of the model.</typeparam>
        /// <typeparam name="TValue">The type of the value.</typeparam>
        /// <param name="html">The HTML helper.</param>
        /// <param name="expression">The expression to evaluate.</param>
        /// <param name="editorTemplate">The editor template to use.</param>
        /// <returns></returns>
        public static MvcHtmlString EditorFor<TModel, TValue>(
            this HtmlHelper<TModel> html,
            Expression<Func<TModel, TValue>> expression,
            EditorTemplate editorTemplate)
        {
            return html.EditorFor(expression, editorTemplate.ToString());
        }

        /// <summary>
        /// Begins the form tag with name as an attribute
        /// </summary>
        /// <param name="htmlHelper">The HTML helper.</param>
        /// <param name="formName">Name of the form.</param>
        /// <returns></returns>
        public static MvcForm BeginForm(this HtmlHelper htmlHelper, string formName)
        {
            return htmlHelper.BeginForm(null, null, FormMethod.Post, new { id = formName,  data_bind="with: model" });
        }
    }
}