﻿var app;
app = (function (app) {

    var disableDescendantsOfControl = function(controlId) {
        var controls = $('#' + controlId).contents();

        controls.find('textarea').attr('disabled', 'disabled');
        controls.find('input').attr('disabled', 'disabled');
        controls.find('select').attr('disabled', 'disabled');
    };
    


    app.ViewModel = function (isInListMode, rootUrl, jsObject, canEdit, rootFormName) {
        
        this.rootUrl = rootUrl;
        if (!isInListMode) {
            this.rootFormName = rootFormName;
            this.canEdit = canEdit;
            this.nextId = 0;
            jsObject.canEdit = canEdit;
            var viewModel = ko.mapping.fromJS(jsObject);
            this.model = ko.observable(viewModel);

            if (!canEdit) {
                disableDescendantsOfControl(rootFormName);
            }
            var self = this;
            window.onbeforeunload = function () {
                if (self.isDirty()) {
                    return 'You have pending changes';
                } else {
                    return undefined;
                }
            };
            ko.applyBindings(self, $('.main-content')[0]);
            this.resetInitialState();
        }

    };
    
    app.ViewModel.prototype.resetInitialState = function () {
        this.initialModelState = ko.mapping.toJSON(this.model());
    };

    app.ViewModel.prototype.isDirty = function () {
        return this.rootUrl !== undefined && ko.mapping.toJSON(this.model()) !== this.initialModelState;
    };
    
    app.ViewModel.prototype.preSave = function () {

    };
    
    app.ViewModel.prototype.getForm = function () {
        return $('#' + this.rootFormName);
    };
    
    app.ViewModel.prototype.save = function () {
        this.preSave();
        var form = this.getForm();
        var saveUrl = this.rootUrl + "Save";
        var self = this;
        form.validate();
        if (form.valid()) {
            self.showGlobalPleaseWait();
            $.ajax({
                url: saveUrl,
                type: "POST",
                data: ko.mapping.toJSON(this.model),
                contentType: 'application/json; charset=utf-8',
                success: function (data) {
                    self.hideGlobalPleaseWait();
                    if (self.processUpdateResult(data)) {
                        self.model(ko.mapping.fromJS(data.Result));
                        $.validator.unobtrusive.parse(form);
                        self.resetInitialState();
                    }
                },
                error: function (req, status, error) {
                    self.hideGlobalPleaseWait();
                    self.handleAjaxError(req, status, error);
                    self.postSave(false);
                },
                complete: function () {
                    self.hideGlobalPleaseWait();
                }
            });
        } else {
            self.showGlobalMessage("Invalid data");
        }
        return false;
    };

    app.ViewModel.prototype.showGlobalPleaseWait = function() {
        $("#pleaseWaitGlobalDiv").dialog({
            resizable: false,
            closeOnEscape: false,
            height: 200,
            width: 400,
            modal: true,
            autoOpen: true,
            buttons: {},
            open: function() { $(".ui-dialog-titlebar-close").hide(); }
        });
    };
    
    app.ViewModel.prototype.hideGlobalPleaseWait = function() {
        $(".ui-dialog-titlebar-close").show();
        if ($('#pleaseWaitGlobalDiv').is(':data(uiDialog)')) {
            $("#pleaseWaitGlobalDiv").dialog('close');
        }
    };
    
    app.ViewModel.prototype.processUpdateResult = function (result) {
        var self = this;
        if (result) {
            var returnValue = true;
            if (result.ValidationError) {
                returnValue = false;
                self.hideGlobalPleaseWait();
                self.showGlobalMessage(result.ValidationError);
            }
            if (result.ConcurrencyError) {
                returnValue = false;
                self.hideGlobalPleaseWait();
                self.showGlobalMessage(result.ConcurrencyError);
            }
            if (result.UnhandledError) {
                returnValue = false;
                self.hideGlobalPleaseWait();
                self.showGlobalMessage(result.UnhandledError);
            }
            return returnValue;
        } else {
            self.hideGlobalPleaseWait();
            self.showGlobalMessage('Error');
            return false;
        }
    };
    
    app.ViewModel.prototype.showGlobalMessage = function (message) {
        var self = this;
        var buttonOk = {};
        // ReSharper restore InconsistentNaming
        buttonOk['OK'] = function () { $(this).dialog("close"); };
        
        $("#globalMessageDiv").dialog({
            resizable: false,
            closeOnEscape: false,
            height: 200,
            width: 400,
            modal: true,
            autoOpen: true,
            buttons: buttonOk,
            open: function () {
                $("#globalMessageDivText").html(
                    self.convertLineFeedToLineBreaks(message));
            }
        });
    };

    app.ViewModel.prototype.convertLineFeedToLineBreaks = function (message) {
        var breakTag = '<br />';
        return (message + '').replace(/([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1' + breakTag + '$2');
    };


    app.ViewModel.prototype.confirmDelete = function (deleteFunction) {
        var buttonsForDelete = {};
       
        buttonsForDelete['Delete'] = function() {
            $(this).dialog("close");
            deleteFunction();
        };
        
        buttonsForDelete['Cancel'] = function () { $(this).dialog("close"); };
        $('#globalConfirmDeleteDiv').dialog({
            resizable: false,
            height: 200,
            width: 400,
            modal: true,
            buttons: buttonsForDelete
        });
    };
    
    app.ViewModel.prototype.delete = function (refreshFunction, id) {
        var saveUrl = this.rootUrl + "DeleteById";
        var self = this;

        self.showGlobalPleaseWait();
        
        $.ajax({
            url: saveUrl,
            type: "POST",
            data: { id: id },
            success: function(data) {
                if (self.processUpdateResult(data)) {
                    refreshFunction();
                }
            },
            error: function(req, status, error) {
                self.hideGlobalPleaseWait();
                self.handleAjaxError(req, status, error);
            },
            complete: function() {
                self.hideGlobalPleaseWait();
            }
        });
        return false;
    };
    
    return app;
})(window.app || { })